Squid Proxy - Coap Howto
========================

Installing the proxy
--------------------

    cd squid-3.1.9
    ./bootstrap.sh
    ./configure --enable-linux-netfilter --disable-strict-error-checking --prefix=/usr/local/squid
    make
    make install
    cp ../squid.conf /usr/local/squid/etc

### Requirements 

* CoAP support uses TProxy kernel feature, available since 2.6.38. 

  in order to have TProxy support you will need the following options enabled into your kernel config
    
    `
    NF_CONNTRACK
    NETFILTER_TPROXY
    NETFILTER_XT_MATCH_SOCKET
    NETFILTER_XT_TARGET_TPROXY
    `
    
* You will also need IPtables >= 1.4.11 [netfilter](http://www.netfilter.org)

* all Squid's build requirements


Configuring Squid
-----------------

### Files

Default installation directory is `/usr/local/squid`

Squid's configuration file is in `$SQUID_DIR/etc/squid.conf`

Executable file can be found into `$SQUID_DIR/sbin/squid`

Logfile are  in `$SQUID_DIR/var/logs/*`

### Effective user

Squid needs to be run as root (otherwise it will not be able to use tproxy kernel's module).
It changes to a different user after the launch. 

User and group have to be set into the config file and the working directory (`/usr/local/squid`) have to be
chowned to the new user

    groupadd squid_group
    useradd -g squid_group squid_user
    chown -R squid_user:squid_group /usr/local/squid

Finally you have to set this user and group into the configfile

    cache_effective_user squid_user
    cache_effective_group squid_group

### ACL

Squid uses (and needs) an Access Control List

Into the configfile before 
    http_access deny all

You need to add the allowed networks

    acl client_network src 50.50.10.0/24
    acl client_network src 2001:123:456:789::/64
    acl server_network dst 50.50.1.0/24
    acl server_network dst 2001:638:709:1234::/64

    http_access allow client_network
    http_access allow server_network

### Ports

You need to setup the correct ports, expecially the one needed for the *transparent proxy*

    http_port 3128
    http_port 3129 tproxy

### Debug messages

Squid has various levels of debug messages (1 to 9).
Messages are written into `$SQUID_DIR/var/logs/cache.log`

There are also 95 categories for debug messages, category 95 has been added for CoAP.
Hence to visualize coap's debug messages you can add 

    debug_options 95,9

to the configfile.

    debug_options ALL,9

shows all debug messages


### Cache

Config file entries that begins with `refresh_pattern` indicates how log a specific content have to be cached

* first column is a regular expression that defines at which URLs a given rule have to be applied
* second column is the minimum *fresh* time for an object
* third column is the percentage of the last modification time in which the object is still fresh
* fourth column is the maximum *fresh* time for an object

You can also disable cache completely setting 

    cache deny all

### coap mapping rules

    coap_url:

    coap_server_port:
    coap_response_timeout:
    coap_max_retransmit:


You can find extensive documentation into
`src/cf.data.pre` squid source file

Important features missing
--------------------------

* PUT and POST method works but no payload is passed to the coap server
* coap_url MUST be an ipv4 or ipv6 address, dns resolution is NOT WORKING
* COAP_BLOCK OPTION is NOT SUPPORTED
* COAP payload is copied into a static buffer

Usage Examples
--------------
### basic usage (external coap server)

You can use coap-proxy with external coap servers like

    coap://vs0.inf.ethz.ch:5863

[californium](http://people.inf.ethz.ch/mkovatsc/californium.php)


    In squid's configfile set

    # user allowed networks
    acl client_network src 147.162.97.0/24 # here use your local network
    acl server_network dst 129.132.252.0/24

    http_access allow client_network
    http_access allow server_network

    coap_url /129.132.252.221/
    coap_server_port 5683

run squid on your localmachine with

    /usr/local/squid $: ./sbin/squid -N 


and setup firefox to use as proxy `localhost:3128`

visit one of the following

    http://129.132.252.221/helloWorld
    http://129.132.252.221/separate
    http://129.132.252.221/timeResource
    http://129.132.252.221/mirror

other resources are not working due to coap_proxy implementation limits


### Simple scenario with transparent proxy

You can also test `coap://vs0.inf.ethz.ch:5863` using TRANSPARENT PROXY option

network configuration will be:

    server # the remote server vs0.inf.ethz.ch
    ipv4 129.132.252.221

    proxy
    ipv4 147.162.97.16/24

    client
    ipv4 192.168.0.2/24

next we have to setup routing and iptables

on client machine

    route add 129.132.252.221 gw 147.162.97.16

on proxy machine 

    sysctl -w net.ipv4.conf.all.forwarding=1

    sudo ip rule add fwmark 1 lookup 100
    sudo ip route add local 0.0.0.0/0 dev lo table 100

    sudo iptables -t mangle -N DIVERT
    sudo iptables -t mangle -A DIVERT -j MARK --set-mark 1
    sudo iptables -t mangle -A DIVERT -j ACCEPT

    sudo iptables -t mangle -A PREROUTING -m socket -j DIVERT
    sudo iptables -t mangle -A PREROUTING -p tcp --dport 80 -j TPROXY --tproxy-mark 1 --on-port 3129

now on client machine you can visit one of the following

    http://129.132.252.221/helloWorld
    http://129.132.252.221/separate
    http://129.132.252.221/timeResource
    http://129.132.252.221/mirror

without setting an explicit proxy

### implementation scenario

In this simple scenario we will install
a coap server into the proxy machine 
and try to reach it using ipv4 and ipv6 from another client

we will use 
[libcoap](http://libcoap.sourceforge.net/)
to implement the coap server and the coap client

`coap-server` and `coap-client` executables can be found into the examples directory of libcoap

network configuration will be:
    server
    ipv4 192.168.0.1/24
    ipv6 2001:123:456:789::1/64

    proxy
    ipv4 192.168.0.1/24
    ipv6 2001:123:456:789::1/64

    client
    ipv4 192.168.0.2/24
    ipv6 2001:123:456:789::2/64

we need to add an option to the configfile

    coap_url /time/ 
    acl client_network src 192.168.0.0/24
    acl client_network src 2001:123:456:789::/64
    acl server_network dst 127.0.0.0/24
    acl server_network dst 2001:638:709:1234::/64

    http_access allow client_network
    http_access allow server_network


where /time/ is a regexp matching the desidered resource(s) that we want to be translated into coap request





start squid proxy with

    root$: /usr/local/squid/sbin/squid -N

than we need to run a coap server 
into the proxy machine

`./coap-server -p 61618` from libcoap examples directory

### Testing the scenario 

Into the client machine fire up a browser and configure it to use 
192.168.0.1 port 3128 as proxy

then try to navigate 

    http://[2001:123:456:789::2]/time 

the request
is translated by squid into `coap://[2001:123:456:789::2]:61618/time`
and the server should respond with a time value


### using TPROXY

Squid proxy supports also *transparent proxy* mode

In this mode if the proxy machine is configured with the appropriate iptables rules
it will automatically intercept requests directed toward the coap and
network translate them.

it will automatically intercept requests directed toward the coap network and
translate them.

    sudo ip -6 rule add fwmark 1 lookup 100
    sudo ip -6 route add local ::/0 dev lo table 100

    sudo ip6tables -t mangle -N DIVERT
    sudo ip6tables -t mangle -A DIVERT -j MARK --set-mark 1
    sudo ip6tables -t mangle -A DIVERT -j ACCEPT

    sudo ip6tables -t mangle -A PREROUTING -m socket -j DIVERT
    sudo ip6tables -t mangle -A PREROUTING -p tcp --dport 80 -j TPROXY --tproxy-mark 1 --on-port 3129


now if you try from the client machine browser
to navigate to 

    http://[2001:123:456:789::2]/time 

without using a proxy, squid will still receive the request and translate the result for you


### more *complex* scenario

we have 3 machines on the same network but for testing purposes 
we will route all the traffic across the proxy machine

network configuration will be:
    server
    ipv4 192.168.0.1/24
    ipv6 2001:123:456:789::3/64

    proxy
    ipv4 192.168.0.1/24
    ipv6 2001:123:456:789::1/64

    client
    ipv4 192.168.0.2/24
    ipv6 2001:123:456:789::2/64


next we need to setup routing tables 

client 

    route -6 add  2001:123:456:789::/64  gw 2001:123:456:789::1

server 
    
    route -6 add  2001:123:456:789::/64  gw 2001:123:456:789::1

proxy
    
    sysctl -w net.ipv6.conf.all.forwarding=1


than we have to setup iptables chains as in the previous example 

    sudo ip -6 rule add fwmark 1 lookup 100
    sudo ip -6 route add local ::/0 dev lo table 100

    sudo ip6tables -t mangle -N DIVERT
    sudo ip6tables -t mangle -A DIVERT -j MARK --set-mark 1
    sudo ip6tables -t mangle -A DIVERT -j ACCEPT

    sudo ip6tables -t mangle -A PREROUTING -m socket -j DIVERT
    sudo ip6tables -t mangle -A PREROUTING -p tcp --dport 80 -j TPROXY --tproxy-mark 1 --on-port 3129

finally on the server run

libcoap's coap-server with
    
    ./coap-server -p 61618

and visit

    http://[2001:123:456:789::3]
