/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Angelo P. Castellani <castellani@dei.unipd.it>
 *  Mattia Gheda <ghedamat@dei.unipd.it>
 */

#ifndef COAP_H
#define COAP_H

#include <stdint.h>
#include <cstring>
#include <netinet/in.h>

#ifndef i386
#define i386
#endif

#ifdef i386
typedef uint8_t nx_uint8_t;
typedef uint16_t nx_uint16_t;
typedef uint32_t nx_uint32_t;
#endif

typedef struct {
    uint8_t addr[16];
} ip6_addr_t;

#define UQ_MULTI_TIMER_P "pippo"
#define COAP_MAX_URI_LEN  40
#define COAP_MAX_PKT 1280

#define UQ_COAP_INSTANCE "PLUTO"

#define COAP_MAX_RESOURCES 5

enum coap_methods_mask {
  COAP_MASK_GET = 0x1,
  COAP_MASK_POST = 0x2,
  COAP_MASK_PUT = 0x4,
  COAP_MASK_DELETE = 0x8 
};

typedef struct {
  uint8_t res_id;
  uint8_t methods;
  char uri[COAP_MAX_URI_LEN]; 
} coap_resource_t;

typedef uint8_t coap_uri_t;
typedef uint8_t coap_char_t;
typedef uint16_t coap_sid_t;

typedef struct {
  ip6_addr_t * host;
  uint16_t port;
  coap_uri_t * uri;
} coap_absuri_t;

// coap token
typedef struct {
  coap_char_t val[8];
  coap_char_t len;
} coap_token_t;

// coap payload 
typedef struct {
  coap_char_t * data;
  uint32_t len;
} coap_payload_t; 


/* 
 * coap packet (packet to buffer?)
 */

typedef struct {
  coap_char_t buf[COAP_MAX_PKT];
  uint32_t len;
} coap_packet_t;

/*
 * coap msg -> coap metadata
 */
typedef struct {
  uint8_t code;
  uint8_t type;
  uint16_t mid;
  uint16_t port;
  uint16_t source_port;
  coap_payload_t * payload;
  coap_token_t token;
  coap_uri_t uri[COAP_MAX_URI_LEN];
  uint8_t format;

  // host port for the lower layer
  ip6_addr_t host;
  // XXX 
} coap_msg_t;

/*
 * Coap Session Struct
 */
typedef uint8_t coap_state_t;
typedef uint8_t coap_status_t;
typedef uint8_t coap_event_t;
typedef uint8_t coap_pkt_t;

typedef struct {
  // host port for the lower layer
  uint16_t port;
  uint16_t source_port;
  uint16_t mid;
  ip6_addr_t host;

  coap_state_t state;
  coap_status_t status;

  uint8_t timeouts;
  uint8_t timer_id;
  coap_token_t token;
  uint8_t idx;
  //coap_msg_t msg; // CACHED MSG
  coap_packet_t pkt;
   
} coap_session_t;

typedef uint8_t coap_client_id_t;
typedef uint8_t coap_session_id_t;

enum coap_session_states{
  SESSION_STATE_IDLE = 1,
  SESSION_STATE_WAIT_ACK,
  SESSION_STATE_WAIT_DEFERRED, 
  SESSION_STATE_RESET,
  SESSION_STATE_WAIT_SEND,
  SESSION_STATE_CACHED,
  SESSION_STATE_WAIT_APP
};   

enum coap_events{
  EVENT_SEND = 1,       
  EVENT_TIMER_FIRED = 4,        
  EVENT_RX = 5           
};

enum coap_session_statuses{
  SESSION_STATUS_NULL = 0,
  SESSION_STATUS_SERVING = 1,
  SESSION_STATUS_CACHED = 2,
  SESSION_STATUS_DEFERRED = 3
};


// this have to be the lowest

#ifndef COAP_TIMEOUT_WAIT_APP 
#define COAP_TIMEOUT_WAIT_APP 200
#endif

// this is a little higher
#ifndef COAP_TIMEOUT_RETRANSMIT
#define COAP_TIMEOUT_RETRANSMIT  300
#endif


// NUM OF RETRANSMIT
#ifndef COAP_CLIENT_TIMEOUTS
#define COAP_CLIENT_TIMEOUTS  0
#endif

#ifndef COAP_TIMEOUT_DEFERRED
#define COAP_TIMEOUT_DEFERRED 15000
#endif

#ifndef COAP_TIMEOUT_WAIT_SEND
#define COAP_TIMEOUT_WAIT_SEND 15000
#endif

void parse_packet( uint8_t * buf, uint32_t len, coap_msg_t * msg);
void write_packet( coap_packet_t * pkt, coap_msg_t * msg);
#include "CoAP-04.h"

#endif
