/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

#include "coap_debug.h"

#include "Debug.h"

void printIpAddress(IpAddress ip, const char *varName, int debugLevel)
{
	debugs(95, debugLevel, "IpAddress " << varName << " = " << ip << " , port = " << ip.GetPort());
}

void printHttpRequest(HttpRequest *request, const char *varName, int debugLevel)
{
	debugs(95, debugLevel, "Instance " << varName << " of class HttpRequest");
	
	debugs(95, debugLevel, "--- host_addr");
	printIpAddress(request->host_addr, "host_addr");
	
	debugs(95, debugLevel, "--- client_addr");
	printIpAddress(request->client_addr, "client_addr");
	
	debugs(95, debugLevel, "--- my_addr");
	printIpAddress(request->my_addr, "my_addr");
	
	debugs(95, debugLevel, "--- host : " << request->GetHost());
	
	debugs(95, debugLevel, "--- port : " << request->port);
	
	debugs(95, debugLevel, "--- method : " << request->method.id() << " = " << request->method.image());
}

void enterFunction(const char *functionName, int debugLevel)
{
	debugs(95, debugLevel, "CoAP >>> Enter in " << functionName);
}

void exitFunction(const char *functionName, int debugLevel)
{
	debugs(95, debugLevel, "CoAP <<< Exit from " << functionName);
}