/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

/**
 * Everything you could need to simplify the debugging of CoAP module.
 */

#ifndef COAP_DEBUG_H
#define COAP_DEBUG_H

#include "ip/IpAddress.h"
#include "HttpRequest.h"

/**
 * Print in the log file the given IP address and its port.
 * @param[in] ip the IP address to visualize
 * @param[in] varName a name to identify the IP address in the log file
 * @param[in] debugLevel a debug level (default = 5)
 */
void printIpAddress(IpAddress ip, const char *varName, int debugLevel = 5);

/**
 * Print in the log file the most useful variables of the given HttpRequest.
 * @param[in] ip a pointer to the istance of HttpRequest to visualize
 * @param[in] varName a name to identify the HttpRequest in the log file
 * @param[in] debugLevel a debug level (default = 9)
 */
void printHttpRequest(HttpRequest *httpRequest, const char *varName, int debugLevel = 9);

/**
 * Mark the entering in a function in the log file.
 * @param[in] functionName a name to identify the function in the log file
 * @param[in] debugLevel a debug level (default = 6)
 */
void enterFunction(const char *functionName, int debugLevel = 6);

/**
 * Mark the exiting from a function in the log file.
 * @param[in] varName a name to identify the function in the log file
 * @param[in] debugLevel a debug level (default = 6)
 */
void exitFunction(const char *functionName, int debugLevel = 6);

#endif /* COAP_DEBUG_H */