/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

#include "coap_module.h"
#include "coap_debug.h"

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <net/if.h>

#include "structs.h"
#include "HttpHeader.h"

/*
void setCoapFlags(FwdState *fwd, const char *url, const char *host)
{
	if (Config.Coap.homogeneous_mapping) // http: == coap:
	{
		IpAddress *test_address;
		// test for IPv4
		test_address = new IpAddress(host);
		printIpAddress(*test_address, "test_address for IPv4");
		if (!test_address->IsAnyAddr()) // IPv4 address
		{
			debugs(95, 5, "host -> Ipv4 address");
			test_address->ApplyMask(Config.Coap.netmask_ipv4);
			if (*test_address == Config.Coap.network_ipv4) // CoAP network
			{
				fwd->towards_coap_network = true;
				fwd->destination_needs_mapping = true;
			}
			delete test_address;
		}
		else
		{
			delete test_address;
			// test for IPv6
			// I try to find and remove eventual square brackets
			int host_length = strlen(host);
			char host_addr[host_length];
			if (strchr(host, '[') != 0) // IPv6 address
			{
				debugs(95, 5, "host -> IPv6 address");
				strncpy(host_addr, host+1, host_length-2);
				host_addr[host_length-2] = 0;
				test_address = new IpAddress(host_addr);
				printIpAddress(*test_address, "test_address for IPv6");
				test_address->ApplyMask(Config.Coap.netmask_ipv6);
				if (*test_address == Config.Coap.network_ipv6) // CoAP network
					fwd->towards_coap_network = true;
				delete test_address;
			}
			else // domain name
			{
				debugs(95, 5, "host -> domain name");
				const char *coap_domain = Config.Coap.coap_domain;
				debugs(95, 9, host + strlen(host) - strlen(coap_domain));
				debugs(95, 9, coap_domain);
				if (0 == strcmp(coap_domain, host + strlen(host) - strlen(coap_domain))) // CoAP network
					fwd->towards_coap_network = true;
			}
		}
	}
	else // http: != coap:
	{
		if (0 == strncmp("coap:", url, 5))
		{
			fwd->towards_coap_network = true;
			IpAddress *test_address;
			// test for IPv4
			test_address = new IpAddress(host);
			printIpAddress(*test_address, "test_address for IPv4");
			if (!test_address->IsAnyAddr()) // IPv4 address
			{
				debugs(95, 5, "host -> Ipv4 address");
				fwd->destination_needs_mapping = true;
			}
			delete test_address;
		}
	}
}
*/

uint16_t newMid()
{
	return (uint16_t)floor(rand() * (double)65535 / (double)RAND_MAX) + 1;
}

/*
IpAddress mapIPv4ToIPv6 (IpAddress address4)
{
	struct addrinfo *address_info6;
	address_info6 = 0;
	Config.Coap.ipv6_mapping_network.GetAddrInfo(address_info6, AF_INET6);
	struct sockaddr_in6 address_socket6;
	memcpy(&address_socket6, address_info6->ai_addr, sizeof(struct sockaddr_in6));
	Config.Coap.ipv6_mapping_network.FreeAddrInfo(address_info6);

	struct addrinfo *address_info4;
	address_info4 = 0;
	address4.GetAddrInfo(address_info4, AF_INET6);
	struct sockaddr_in6 address_socket4;
	memcpy(&address_socket4, address_info4->ai_addr, sizeof(struct sockaddr_in6));
	address4.FreeAddrInfo(address_info4);

	for (int i = 0; i < 12; i++)
		address_socket4.sin6_addr.s6_addr[i] = address_socket6.sin6_addr.s6_addr[i];
	
	IpAddress address6(address_socket4);
	return address6;
}
*/

bool buildCoapRequest(HttpRequest *request, MemBuf *buffer, uint16_t mid)
{
	enterFunction("buildCoapRequest");
	
	debugs(95, 7, "1. Initializations");
	coap_packet_t *pkt;
	coap_packet_constructor(&pkt);
	
	debugs(95, 7, "2. Method");
	uint8_t method = 0;
	if (strcmp(request->method.image(), "GET") == 0)
		method = COAP_METHOD_GET;
	else if (strcmp(request->method.image(), "POST") == 0)
		method = COAP_METHOD_POST;
	else if (strcmp(request->method.image(), "PUT") == 0)
		method = COAP_METHOD_PUT;
	else if (strcmp(request->method.image(), "DELETE") == 0)
		method = COAP_METHOD_DELETE;
	else
	{
		debugs(95, 5, "Unsupported method");
		return false;
	}
	// FIXME: support POST and PUT
	
	debugs(95, 7, "4. Writing packet");
	/*
	void coap_create_packet (	coap_packet_t *pkt,
								uint8_t packet_type,
								uint8_t packet_code,
								uint16_t message_id,
								char *uri_path,
								char *content_data,
								uint8_t content_length,
								uint8_t content_type	);
	*/
	coap_create_packet (pkt, COAP_T_CON, method, mid, request->urlpath.termedBuf(), "", 0, 0);
	debugs(95,7,"url " << request->urlpath.termedBuf());
	// FIXME: options uri_host and uri_port can't be empty in a request
	// FIXME: uri_host can be IPv4, IPv6 or a domain name
	
	debugs(95, 7, "5. Writing buffer");
	buffer->reset();
	buffer->append((char *)pkt->buf, pkt->len);
	
	debugs(95, 7, "6. Deinitializations");
	coap_packet_destructor(&pkt);
	
	exitFunction("buildCoapRequest");
	return true;
}

void buildCoapEmptyAck(coap_packet_t *pkt, uint16_t mid)
{
	enterFunction("buildCoapEmptyAck");
	
	/*
	 * void coap_create_packet (	coap_packet_t *pkt,
	 *							uint8_t packet_type,
	 *							uint8_t packet_code,
	 *							uint16_t message_id,
	 *							char *rel_uri,
	 *							char *content_data,
	 *							uint8_t content_length,
	 *							uint8_t content_type	);
	 */
	coap_create_packet (pkt, COAP_T_ACK, 0, mid, "", "", 0, 0);
	
	exitFunction("buildCoapEmptyAck");
}

void translateCoapToHttp(coap_msg_t *msg, MemBuf *buffer)
{
	enterFunction("translateCoapToHttp");
	
	debugs(95, 7, "1. CoAP Response code " << msg->code << "");

	debugs(95, 7, "1. HTTP Response code");
	char httpPacket[4096];
	strcpy(httpPacket, "HTTP/1.0 ");
	switch (msg->code)
	{
		case COAP_STATUS_OK:
			strcat(httpPacket, "200 OK");
			break;
		case COAP_STATUS_OK_CREATED:
			strcat(httpPacket, "201 Created");
			break;
		case COAP_STATUS_OK_DELETED:
			strcat(httpPacket, "204 No Content");
			break;
		case COAP_STATUS_OK_VALID:
			strcat(httpPacket, "304 Not Modified");
			break;
		case COAP_STATUS_OK_CHANGED:
			strcat(httpPacket, "204 No Content");
			break;
		case COAP_STATUS_OK_CONTENT:
			strcat(httpPacket, "200 OK");
			break;
		case COAP_STATUS_BAD_REQUEST:
			strcat(httpPacket, "400 Bad Request");
			break;
		case COAP_STATUS_UNAUTHORIZED:
			strcat(httpPacket, "401 Unauthorized");
			break;
		case COAP_STATUS_BAD_OPTION:
			strcat(httpPacket, "400 Bad Request");
			break;
		case COAP_STATUS_FORBIDDEN:
			strcat(httpPacket, "403 Forbidden");
			break;
		case COAP_STATUS_NOT_FOUND:
			strcat(httpPacket, "404 Not Found");
			break;
		case COAP_STATUS_METHOD_NOT_ALLOWED:
			strcat(httpPacket, "405 Method Not Allowed");
			break;
		case COAP_STATUS_REQUEST_ENTITY_TOO_LARGE:
			strcat(httpPacket, "413 Request Entity Too Large");
			break;
		case COAP_STATUS_UNSUPPORTED_MEDIA_TYPE:
			strcat(httpPacket, "415 Unsupported Media Type");
			break;
		case COAP_STATUS_INTERNAL_SERVER_ERROR:
			strcat(httpPacket, "500 Internal Server Error");
			break;
		case COAP_STATUS_NOT_IMPLEMENTED:
			strcat(httpPacket, "501 Not Implemented");
			break;
		case COAP_STATUS_BAD_GATEWAY:
			strcat(httpPacket, "502 Bad Gateway");
			break;
		case COAP_STATUS_SERVICE_UNAVAILABLE:
			strcat(httpPacket, "503 Service Unavailable");
			break;
		case COAP_STATUS_GATEWAY_TIMEOUT:
			strcat(httpPacket, "504 Gateway Timeout");
			break;
		case COAP_STATUS_PROXYING_NOT_SUPPORTED:
			strcat(httpPacket, "502 Bad Gateway");
			break;
		default:
			strcat(httpPacket, "502 Bad Gateway");
	}
	strcat(httpPacket, "\r\n");
	
	debugs(95, 7, "2. Content length");
	strcat(httpPacket, "Content-Length: ");
	char tempString[20];
	sprintf(tempString, "%d", msg->payload->len);
	strcat(httpPacket, tempString);
	strcat(httpPacket, "\r\n");
	
	debugs(95, 7, "3. Content type");
	strcat(httpPacket, "Content-Type: ");
	switch (msg->format)
	{
		case COAP_CONTENTTYPE_PLAIN:
			strcat(httpPacket, "text/plain; charset=utf-8");
			break;
		case COAP_CONTENTTYPE_EXI:
			strcat(httpPacket, "application/exi");
			break;
		case 3: // COAP_CONTENTTYPE_HTML
			strcat(httpPacket, "text/html; charset=utf-8");
			break;
		default:
			// TODO: other codes
			strcat(httpPacket, "text/plain; charset=utf-8");
	}
	strcat(httpPacket, "\r\n");
	
	debugs(95, 7, "5. Max age");
	strcat(httpPacket, "Cache-Control: max-age=60\r\n");
	// TODO: implement option max-age
	
	strcat(httpPacket, "Connection: close\r\n");
	
	debugs(95, 7, "4. Data");
	strcat(httpPacket, "\r\n");
	strncat(httpPacket, (char *)msg->payload->data, msg->payload->len);
	
	debugs(95, 7, "5. Writing packet into buffer");
	buffer->reset();
	buffer->append(httpPacket, strlen(httpPacket));
	
	exitFunction("translateCoapToHttp");
}
