/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

/** These functions are parts of code relative to CoAP that can be
 *  isolated from the Squid code.
 */

#ifndef COAP_MODULE_H
#define COAP_MODULE_H

#include <stdint.h>

#include "ip/IpAddress.h"
#include "HttpRequest.h"
#include "MemBuf.h"
#include "forward.h"

#include "coap_utils.h"

/**
 * setCoapFlags
 */
/* void setCoapFlags(FwdState *fwd, const char *url, const char *host); */

/**
 * Generate a new message ID.
 * @return a random message ID
 */
uint16_t newMid();

/**
 * mapIPv4ToIPv6
 */
/*
IpAddress mapIPv4ToIPv6 (IpAddress address4);
*/

/**
 * Builds a CoAP packet from an HTTP request, and writes it in the given buffer.
 * @param[in] request pointer to the HTTP request to take informations from
 * @param[out] buffer pointer to the buffer to write the CoAP packet in
 * @param[in] mid the message ID of the packet that will be created
 * @return true if no errors, false if something in the conversion is not supported
 */
bool buildCoapRequest(HttpRequest *request, MemBuf *buffer, uint16_t mid);

/**
 * Builds a CoAP empty ack, and writes int in the given coap_packet_t.
 * @param[out] pkt pointer to the coap_packet_t to write the packet in
 * @param[in] mid the message ID of the packet thata will be created
 */
void buildCoapEmptyAck(coap_packet_t *pkt, uint16_t mid);

/**
 * Builds an HTTP reply packet from the informations in coap_msg_t
 * @param[in] msg pointer to the coap_msg_t to take the informations from
 * @param[out] buffer pointer to the buffer to write the HTTP packet in
 */
void translateCoapToHttp(coap_msg_t *msg, MemBuf *buffer);

#endif /* COAP_MODULE_H */
