/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

#include "coap_utils.h"

void coap_packet_constructor (coap_packet_t **pkt)
{
	*pkt = (coap_packet_t *)malloc(sizeof(coap_packet_t));
	memset(*pkt, 0, sizeof(coap_packet_t));
}

void coap_packet_destructor (coap_packet_t **pkt)
{
	free(*pkt);
	*pkt = 0;
}

void coap_message_constructor (coap_msg_t **msg)
{
	*msg = (coap_msg_t *)malloc(sizeof(coap_msg_t));
	memset(*msg, 0, sizeof(coap_msg_t));
	(*msg)->payload = (coap_payload_t *)malloc(sizeof(coap_payload_t));
	(*msg)->payload->data = (coap_char_t *)calloc(COAP_MAX_PKT, sizeof(coap_char_t));
	(*msg)->payload->len = 0;
}

void coap_message_destructor (coap_msg_t **msg)
{
	free((*msg)->payload->data);
	free((*msg)->payload);
	free(*msg);
	*msg = 0;
}

void coap_create_packet (	coap_packet_t *pkt,
							uint8_t packet_type,
							uint8_t packet_code,
							uint16_t message_id,
							const char *uri_path,
							const char *content_data,
							uint8_t content_length,
							uint8_t content_type	)
{
	// Initializations
	coap_msg_t *msg;
	coap_message_constructor(&msg);
	// Coap packet
	msg->code = packet_code;
	msg->type = packet_type;
	msg->mid = message_id;
	// Payload
	if (content_length > COAP_MAX_PKT)
		content_length = COAP_MAX_PKT;
	memcpy(msg->payload->data, content_data, content_length);
	msg->payload->len = content_length;
	msg->format = content_type;
	// Uri
	strncpy((char *)msg->uri, uri_path, COAP_MAX_URI_LEN);
	// Writing
	write_packet(pkt, msg);
	// Deinitializations
	coap_message_destructor(&msg);
}

void coap_parse_packet (	coap_msg_t *msg,
							char *buffer,
							uint32_t buffer_length	)
{
	// Initializations
	coap_packet_t *pkt;
	coap_packet_constructor(&pkt);
	// Packet
	memcpy(pkt->buf, buffer, buffer_length);
	pkt->len = buffer_length;
	// Parsing
	parse_packet(pkt->buf, buffer_length, msg);
	// Deinitializations
	coap_packet_destructor(&pkt);
}
