/* Copyright (c) 2009 The Regents of the Signet Group.
 *  All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *  Authors:
 *  Simone Pinton <simone.pinton@gmail.com>
 */

/**
 * A simpler interface for the CoAP module.
 */

#ifndef COAP_UTILS_H
#define COAP_UTILS_H

#include <cstdlib>
#include <cstring>
#include <stdint.h>
#include <netinet/in.h>

#include "CoAP.h"

/**
 * Allocate a coap_packet_t and all its nested structures.
 * Initializes all variables to 0.
 * @param[out] pkt a pointer to a pointer of a coap_packet_t
 */
void coap_packet_constructor(coap_packet_t **pkt);

/**
 * Deallocate a coap_packet_t and all its nested structures.
 * Returns a null pointer.
 * @param[in] pkt a pointer to the the pointer of the coap_packet_t to deallocate
 */
void coap_packet_destructor(coap_packet_t **pkt);

/**
 * Allocate a coap_msg_t and all its nested structures.
 * Initializes all variables to 0.
 * @param[out] msg a pointer to a pointer of a coap_msg_t
 */
void coap_message_constructor(coap_msg_t **msg);

/**
 * Deallocate a coap_msg_t and all its nested structures.
 * Returns a null pointer.
 * @param[in] msg a pointer to the pointer of the coap_msg_t to deallocate
 */
void coap_message_destructor(coap_msg_t **msg);

/**
 * Fill coap_packet_t using the given infos.
 * @param[out] pkt a pointer to the coap_packet_t to fill
 * @param[in] packet_type CoAP packet type (from enum coap_t_values)
 * @param[in] packet_code CoAP packet code (0 if empty ack, or from enum coap_method_values if request, or from enum coap_status_values if reply)
 * @param[in] message_id CoAP message ID
 * @param[in] uri_path URI path //FIXME: empty
 * @param[in] content_data packet payload //FIXME: empty
 * @param[in] content_length payload length
 * @param[in] content_type payload content type (from enum coap_contenttype_values)
 */
void coap_create_packet (	coap_packet_t *pkt,
							uint8_t packet_type,
							uint8_t packet_code,
							uint16_t message_id,
							const char *uri_path,
							const char *content_data,
							uint8_t content_length,
							uint8_t content_type	);

/**
 * Fill coap_msg_t using the given buffer containing a CoAP packet.
 * @param[out] pkt a pointer to the coap_msg_t to fill
 * @param[in] buffer the buffer containing the CoAP packet to be parsed
 * @param[in] buffer_length the buffer length
 */
void coap_parse_packet (	coap_msg_t *msg,
							char *buffer,
							uint32_t buffer_length	);

#endif
